@@ -48,7 +48,7 @@ mod keyed_elements;
 ///     diff,
 ///     vec![RemoveNode::new(
 ///         Some(&"div"),
-///         TreePath::start_at(1, vec![0, 0]),
+///         TreePath::new(vec![0, 0]),
 ///     )
 ///     .into()]
 /// );
@@ -67,9 +67,6 @@ where
     diff_recursive(
         old_node,
         new_node,
-        &mut 0,
-        &mut 0,
-        &vec![0],
         &vec![0],
         key,
         &|_old, _new| false,
@@ -104,17 +101,7 @@ where
     SKIP: Fn(&'a Node<NS, TAG, ATT, VAL>, &'a Node<NS, TAG, ATT, VAL>) -> bool,
     REP: Fn(&'a Node<NS, TAG, ATT, VAL>, &'a Node<NS, TAG, ATT, VAL>) -> bool,
 {
-    diff_recursive(
-        old_node,
-        new_node,
-        &mut 0,
-        &mut 0,
-        &vec![0],
-        &vec![0],
-        key,
-        skip,
-        rep,
-    )
+    diff_recursive(old_node, new_node, &vec![0], key, skip, rep)
 }
 
 /// returns true if any of the node children has key in their attributes
@@ -202,10 +189,7 @@ where
 fn diff_recursive<'a, 'b, NS, TAG, ATT, VAL, SKIP, REP>(
     old_node: &'a Node<NS, TAG, ATT, VAL>,
     new_node: &'a Node<NS, TAG, ATT, VAL>,
-    cur_node_idx: &'b mut usize,
-    new_node_idx: &'b mut usize,
     cur_path: &Vec<usize>,
-    new_path: &Vec<usize>,
     key: &ATT,
     skip: &SKIP,
     rep: &REP,
@@ -220,8 +204,6 @@ where
 {
     // skip diffing if the function evaluates to true
     if skip(old_node, new_node) {
-        *cur_node_idx += old_node.descendant_node_count();
-        *new_node_idx += new_node.descendant_node_count();
         return vec![];
     }
 
@@ -233,13 +215,11 @@ where
         patches.push(
             ReplaceNode::new(
                 old_node.tag(),
-                TreePath::start_at(*cur_node_idx, cur_path.clone()),
+                TreePath::new(cur_path.clone()),
                 new_node,
             )
             .into(),
         );
-        *cur_node_idx += old_node.descendant_node_count();
-        *new_node_idx += new_node.descendant_node_count();
         return patches;
     }
 
@@ -252,7 +232,7 @@ where
             if old_text != new_text {
                 let ct = ChangeText::new(
                     old_text,
-                    TreePath::start_at(*cur_node_idx, cur_path.clone()),
+                    TreePath::new(cur_path.clone()),
                     new_text,
                 );
                 patches.push(Patch::ChangeText(ct));
@@ -270,10 +250,7 @@ where
                     old_element,
                     new_element,
                     key,
-                    cur_node_idx,
-                    new_node_idx,
                     cur_path,
-                    new_path,
                     skip,
                     rep,
                 );
@@ -283,10 +260,7 @@ where
                     old_element,
                     new_element,
                     key,
-                    cur_node_idx,
-                    new_node_idx,
                     cur_path,
-                    new_path,
                     skip,
                     rep,
                 );
@@ -318,10 +292,7 @@ fn diff_non_keyed_elements<'a, 'b, NS, TAG, ATT, VAL, SKIP, REP>(
     old_element: &'a Element<NS, TAG, ATT, VAL>,
     new_element: &'a Element<NS, TAG, ATT, VAL>,
     key: &ATT,
-    cur_node_idx: &'b mut usize,
-    new_node_idx: &'b mut usize,
     cur_path: &Vec<usize>,
-    new_path: &Vec<usize>,
     skip: &SKIP,
     rep: &REP,
 ) -> Vec<Patch<'a, NS, TAG, ATT, VAL>>
@@ -333,15 +304,10 @@ where
     SKIP: Fn(&'a Node<NS, TAG, ATT, VAL>, &'a Node<NS, TAG, ATT, VAL>) -> bool,
     REP: Fn(&'a Node<NS, TAG, ATT, VAL>, &'a Node<NS, TAG, ATT, VAL>) -> bool,
 {
-    let this_cur_node_idx = *cur_node_idx;
     let this_cur_path = cur_path.clone();
     let mut patches = vec![];
-    let attributes_patches = create_attribute_patches(
-        old_element,
-        new_element,
-        *cur_node_idx,
-        cur_path,
-    );
+    let attributes_patches =
+        create_attribute_patches(old_element, new_element, cur_path);
     patches.extend(attributes_patches);
 
     let old_child_count = old_element.children.len();
@@ -349,17 +315,8 @@ where
 
     let min_count = cmp::min(old_child_count, new_child_count);
     for index in 0..min_count {
-        *cur_node_idx += 1;
-        *new_node_idx += 1;
-
         let mut cur_child_path = cur_path.clone();
-        let mut new_child_path = new_path.clone();
         cur_child_path.push(index);
-        println!(
-            "\t we just added index: {} ... cur_child_path is now: {:?}",
-            index, cur_child_path
-        );
-        new_child_path.push(index);
 
         let old_child = &old_element
             .children
@@ -371,10 +328,7 @@ where
         let more_patches = diff_recursive(
             old_child,
             new_child,
-            cur_node_idx,
-            new_node_idx,
-            &mut cur_child_path,
-            &mut new_child_path,
+            &cur_child_path,
             key,
             skip,
             rep,
@@ -388,20 +342,14 @@ where
         let append_children_patch = create_append_children_patch(
             old_element,
             new_element,
-            this_cur_node_idx,
-            new_node_idx,
             this_cur_path,
         );
         patches.push(append_children_patch);
     }
 
     if new_child_count < old_child_count {
-        let remove_node_patches = create_remove_node_patch(
-            old_element,
-            new_element,
-            cur_node_idx,
-            cur_path,
-        );
+        let remove_node_patches =
+            create_remove_node_patch(old_element, new_element, cur_path);
         patches.extend(remove_node_patches);
     }
 
@@ -411,8 +359,6 @@ where
 fn create_append_children_patch<'a, 'b, NS, TAG, ATT, VAL>(
     old_element: &'a Element<NS, TAG, ATT, VAL>,
     new_element: &'a Element<NS, TAG, ATT, VAL>,
-    this_cur_node_idx: usize,
-    new_node_idx: &'b mut usize,
     this_cur_path: Vec<usize>,
 ) -> Patch<'a, NS, TAG, ATT, VAL>
 where
@@ -422,17 +368,15 @@ where
     VAL: PartialEq + Clone + Debug,
 {
     let old_child_count = old_element.children.len();
-    let mut append_patch: Vec<(usize, &'a Node<NS, TAG, ATT, VAL>)> = vec![];
+    let mut append_patch: Vec<&'a Node<NS, TAG, ATT, VAL>> = vec![];
 
     for append_child in new_element.children.iter().skip(old_child_count) {
-        *new_node_idx += 1;
-        append_patch.push((*new_node_idx, append_child));
-        *new_node_idx += append_child.descendant_node_count();
+        append_patch.push(append_child);
     }
 
     AppendChildren::new(
         &old_element.tag,
-        TreePath::start_at(this_cur_node_idx, this_cur_path.clone()),
+        TreePath::new(this_cur_path.clone()),
         append_patch,
     )
     .into()
@@ -441,7 +385,6 @@ where
 fn create_remove_node_patch<'a, 'b, NS, TAG, ATT, VAL>(
     old_element: &'a Element<NS, TAG, ATT, VAL>,
     new_element: &'a Element<NS, TAG, ATT, VAL>,
-    cur_node_idx: &'b mut usize,
     cur_path: &Vec<usize>,
 ) -> Vec<Patch<'a, NS, TAG, ATT, VAL>>
 where
@@ -458,20 +401,15 @@ where
         .skip(new_child_count)
         .enumerate()
     {
-        *cur_node_idx += 1;
         let mut child_cur_path = cur_path.clone();
         child_cur_path.push(new_child_count + i);
-        let remove_node_patch = RemoveNode::new(
-            old_child.tag(),
-            TreePath::start_at(*cur_node_idx, child_cur_path),
-        );
+        let remove_node_patch =
+            RemoveNode::new(old_child.tag(), TreePath::new(child_cur_path));
         patches.push(remove_node_patch.into());
-        *cur_node_idx += old_child.descendant_node_count();
     }
     patches
 }
 
-/// diff the attributes of old_node element to the new element at this cur_node_idx
 ///
 /// Note: The performance bottlenecks
 ///     - allocating new vec
@@ -479,7 +417,6 @@ where
 fn create_attribute_patches<'a, 'b, NS, TAG, ATT, VAL>(
     old_element: &'a Element<NS, TAG, ATT, VAL>,
     new_element: &'a Element<NS, TAG, ATT, VAL>,
-    cur_node_idx: usize,
     cur_path: &Vec<usize>,
 ) -> Vec<Patch<'a, NS, TAG, ATT, VAL>>
 where
@@ -545,7 +482,7 @@ where
         patches.push(
             AddAttributes::new(
                 &old_element.tag,
-                TreePath::start_at(cur_node_idx, cur_path.clone()),
+                TreePath::new(cur_path.clone()),
                 add_attributes,
             )
             .into(),
@@ -555,7 +492,7 @@ where
         patches.push(
             RemoveAttributes::new(
                 &old_element.tag,
-                TreePath::start_at(cur_node_idx, cur_path.clone()),
+                TreePath::new(cur_path.clone()),
                 remove_attributes,
             )
             .into(),
