@@ -41,13 +41,10 @@ where
 fn find_matched_new_child<'a, NS, TAG, ATT, VAL>(
     matched_old_new_keyed: &BTreeMap<
         (usize, usize),
-        (
-            &'a Node<NS, TAG, ATT, VAL>,
-            (usize, &'a Node<NS, TAG, ATT, VAL>),
-        ),
+        (&'a Node<NS, TAG, ATT, VAL>, &'a Node<NS, TAG, ATT, VAL>),
     >,
     find_old_idx: usize,
-) -> Option<(usize, (usize, &'a Node<NS, TAG, ATT, VAL>))>
+) -> Option<(usize, &'a Node<NS, TAG, ATT, VAL>)>
 where
     NS: PartialEq + Clone + Debug,
     TAG: PartialEq + Clone + Debug,
@@ -89,10 +86,7 @@ where
 fn get_matched_old_new_idx<'a, NS, TAG, ATT, VAL>(
     matched_old_new_keyed: &BTreeMap<
         (usize, usize),
-        (
-            &'a Node<NS, TAG, ATT, VAL>,
-            (usize, &'a Node<NS, TAG, ATT, VAL>),
-        ),
+        (&'a Node<NS, TAG, ATT, VAL>, &'a Node<NS, TAG, ATT, VAL>),
     >,
 ) -> (Vec<usize>, Vec<usize>)
 where
@@ -153,14 +147,11 @@ fn build_matched_old_new_keyed<'a, NS, TAG, ATT, VAL>(
     >,
     new_keyed_elements: &BTreeMap<
         usize,
-        (Vec<&'a VAL>, (usize, &'a Node<NS, TAG, ATT, VAL>)),
+        (Vec<&'a VAL>, &'a Node<NS, TAG, ATT, VAL>),
     >,
 ) -> BTreeMap<
     (usize, usize),
-    (
-        &'a Node<NS, TAG, ATT, VAL>,
-        (usize, &'a Node<NS, TAG, ATT, VAL>),
-    ),
+    (&'a Node<NS, TAG, ATT, VAL>, &'a Node<NS, TAG, ATT, VAL>),
 >
 where
     NS: PartialEq + Clone + Debug,
@@ -197,26 +188,6 @@ where
     matched_old_new_keyed
 }
 
-fn build_node_idx_new_elements<'a, NS, TAG, ATT, VAL>(
-    new_element: &'a Element<NS, TAG, ATT, VAL>,
-    new_node_idx: &mut usize,
-) -> BTreeMap<usize, &'a Node<NS, TAG, ATT, VAL>>
-where
-    NS: PartialEq + Clone + Debug,
-    TAG: PartialEq + Clone + Debug,
-    ATT: PartialEq + Clone + Debug,
-    VAL: PartialEq + Clone + Debug,
-{
-    let mut node_idx_new_elements = BTreeMap::new();
-
-    for new_child in new_element.get_children().iter() {
-        *new_node_idx += 1;
-        node_idx_new_elements.insert(*new_node_idx, new_child);
-        *new_node_idx += new_child.descendant_node_count();
-    }
-    node_idx_new_elements
-}
-
 /// Reconciliation of keyed elements
 ///
 /// algorithm:
@@ -235,10 +206,7 @@ pub fn diff_keyed_elements<'a, 'b, NS, TAG, ATT, VAL, SKIP, REP>(
     old_element: &'a Element<NS, TAG, ATT, VAL>,
     new_element: &'a Element<NS, TAG, ATT, VAL>,
     key: &ATT,
-    cur_node_idx: &'b mut usize,
-    new_node_idx: &'b mut usize,
     cur_path: &Vec<usize>,
-    new_path: &Vec<usize>,
     skip: &SKIP,
     rep: &REP,
 ) -> Vec<Patch<'a, NS, TAG, ATT, VAL>>
@@ -255,22 +223,7 @@ where
     // create a map for both the old and new element
     // we can not use VAL as the key, since it is not Hash
     let old_keyed_elements = build_keyed_elements(old_element, key);
-
-    let node_idx_new_elements =
-        build_node_idx_new_elements(new_element, new_node_idx);
-
-    let new_keyed_elements: BTreeMap<
-        usize,
-        (Vec<&VAL>, (usize, &Node<NS, TAG, ATT, VAL>)),
-    > = BTreeMap::from_iter(
-        node_idx_new_elements.iter().enumerate().filter_map(
-            |(new_idx, (new_element_node_idx, new_child))| {
-                new_child.get_attribute_value(key).map(|new_key| {
-                    (new_idx, (new_key, (*new_element_node_idx, *new_child)))
-                })
-            },
-        ),
-    );
+    let new_keyed_elements = build_keyed_elements(new_element, key);
 
     // compiles that matched old and new with
     // with their (old_idx, new_idx) as key and the value is (old_element, new_element)
@@ -282,16 +235,12 @@ where
         get_matched_old_new_idx(&matched_old_new_keyed);
 
     // these are the new children that didn't matched in the keyed elements pass
-    let mut unmatched_new_child: Vec<(
-        usize,
-        (usize, &'a Node<NS, TAG, ATT, VAL>),
-    )> = vec![];
+    let mut unmatched_new_child: Vec<(usize, &'a Node<NS, TAG, ATT, VAL>)> =
+        vec![];
 
-    for (idx, (node_idx, new_element)) in
-        node_idx_new_elements.iter().enumerate()
-    {
+    for (idx, new_element) in new_element.get_children().iter().enumerate() {
         if !matched_new_idx.contains(&idx) {
-            unmatched_new_child.push((idx, (*node_idx, new_element)));
+            unmatched_new_child.push((idx, new_element));
         }
     }
 
@@ -304,10 +253,7 @@ where
     // matched old and new element, not necessarily keyed
     let matched_old_new: BTreeMap<
         (usize, usize),
-        (
-            &'a Node<NS, TAG, ATT, VAL>,
-            (usize, &'a Node<NS, TAG, ATT, VAL>),
-        ),
+        (&'a Node<NS, TAG, ATT, VAL>, &'a Node<NS, TAG, ATT, VAL>),
     > = BTreeMap::from_iter(
         // try to match unmatched new child from the unmatched old child
         // using the their idx, most likely aligned nodes (ie: old and new node in the same index)
@@ -345,20 +291,12 @@ where
         usize,
         &Node<NS, TAG, ATT, VAL>,
     )> = vec![];
-    for (idx, (new_element_node_idx, new_child)) in
-        node_idx_new_elements.iter().enumerate()
-    {
+    for (idx, new_child) in new_element.get_children().iter().enumerate() {
         if !matched_new_idx_pass2.contains(&idx) {
-            unmatched_new_child_pass2.push((
-                idx,
-                *new_element_node_idx,
-                new_child,
-            ));
+            unmatched_new_child_pass2.push((idx, idx, new_child));
         }
     }
 
-    // process this last so as not to move the cur_node_idx forward
-    // and without creating a snapshot for cur_node_idx for other patch types
     let mut matched_keyed_element_patches = vec![];
     let mut remove_node_patches = vec![];
     let mut insert_node_patches = vec![];
@@ -366,19 +304,11 @@ where
     // keeps track of new_idx that is part of the InsertNode
     let mut already_inserted = vec![];
 
-    // cur_node_idx before incrementing inside the old element iteration and recursive increment
-    // that may be called for each of this old element children nodes
-    let snapshot_cur_node_idx = *cur_node_idx;
-
     for (old_idx, old_child) in old_element.children.iter().enumerate() {
-        *cur_node_idx += 1;
         let mut child_cur_path = cur_path.clone();
         child_cur_path.push(old_idx);
 
-        let mut child_new_path = new_path.clone();
-        child_new_path.push(old_idx);
-
-        if let Some((new_idx, (mut new_child_node_idx, new_child))) =
+        if let Some((new_idx, new_child)) =
             find_matched_new_child(&all_matched_elements, old_idx)
         {
             // insert unmatched new_child that is less than the matched new_idx
@@ -386,7 +316,6 @@ where
                 old_element,
                 &mut already_inserted,
                 &unmatched_new_child_pass2,
-                *cur_node_idx,
                 new_idx,
                 &child_cur_path,
             ));
@@ -394,10 +323,7 @@ where
             matched_keyed_element_patches.extend(diff_recursive(
                 old_child,
                 new_child,
-                cur_node_idx,
-                &mut new_child_node_idx,
                 &child_cur_path,
-                &child_new_path,
                 key,
                 skip,
                 rep,
@@ -406,11 +332,10 @@ where
             remove_node_patches.push(
                 RemoveNode::new(
                     old_child.tag(),
-                    TreePath::start_at(*cur_node_idx, child_cur_path.clone()),
+                    TreePath::new(child_cur_path.clone()),
                 )
                 .into(),
             );
-            *cur_node_idx += old_child.descendant_node_count();
         }
     }
 
@@ -421,15 +346,10 @@ where
         &mut already_inserted,
         &unmatched_new_child_pass2,
         &cur_path,
-        snapshot_cur_node_idx,
     );
 
-    let attributes_patches = create_attribute_patches(
-        old_element,
-        new_element,
-        *cur_node_idx,
-        cur_path,
-    );
+    let attributes_patches =
+        create_attribute_patches(old_element, new_element, cur_path);
 
     // patch order matters here
     // apply changes to the matched element first,
@@ -450,7 +370,6 @@ fn create_insert_node_patches<'a, NS, TAG, ATT, VAL>(
         usize,
         &'a Node<NS, TAG, ATT, VAL>,
     )>,
-    cur_node_idx: usize,
     new_idx: usize,
     child_cur_path: &Vec<usize>,
 ) -> Vec<Patch<'a, NS, TAG, ATT, VAL>>
@@ -469,7 +388,7 @@ where
             insert_node_patches.push(
                 InsertNode::new(
                     Some(&old_element.tag),
-                    TreePath::start_at(cur_node_idx, child_cur_path.clone()),
+                    TreePath::new(child_cur_path.clone()),
                     unmatched,
                 )
                 .into(),
@@ -490,7 +409,6 @@ fn create_append_children_patches<'a, NS, TAG, ATT, VAL>(
         &'a Node<NS, TAG, ATT, VAL>,
     )>,
     cur_path: &Vec<usize>,
-    snapshot_cur_node_idx: usize,
 ) -> Vec<Patch<'a, NS, TAG, ATT, VAL>>
 where
     NS: PartialEq + Clone + Debug,
@@ -499,15 +417,15 @@ where
     VAL: PartialEq + Clone + Debug,
 {
     let mut append_children_patches = vec![];
-    for (new_idx, new_element_node_idx, new_child) in
+    for (new_idx, _new_element_node_idx, new_child) in
         unmatched_new_child_pass2.iter()
     {
         if !already_inserted.contains(&new_idx) {
             append_children_patches.push(
                 AppendChildren::new(
                     &old_element.tag,
-                    TreePath::start_at(snapshot_cur_node_idx, cur_path.clone()),
-                    vec![(*new_element_node_idx, new_child)],
+                    TreePath::new(cur_path.clone()),
+                    vec![new_child],
                 )
                 .into(),
             );
