@@ -37,6 +37,9 @@ where
 pub struct Text {
     /// text node value
     pub text: String,
+    /// if true, then this text content will be injected as inner_html, otherwise it will be a text
+    /// node
+    pub safe_html: bool,
 }
 
 impl Text {
@@ -44,6 +47,16 @@ impl Text {
     pub fn new(txt: impl ToString) -> Self {
         Text {
             text: txt.to_string(),
+            safe_html: false,
+        }
+    }
+
+    /// Create a text node which is intended to be used as inner html for its
+    /// parent node
+    pub fn safe_html(txt: impl ToString) -> Self {
+        Text {
+            text: txt.to_string(),
+            safe_html: true,
         }
     }
 
@@ -236,23 +249,24 @@ where
         }
     }
 
-    /// recursive count the number of nodes under this tree
+    /// Returns the total number of nodes on this node tree, that is counting the direct and
+    /// indirect child nodes of this node.
     pub fn node_count(&self) -> usize {
-        let mut current = 0;
-        self.node_count_recursive(&mut current);
-        current
+        1 + self.descendant_node_count()
     }
 
-    fn node_count_recursive(&self, current: &mut usize) {
+    /// only count the descendant node
+    pub fn descendant_node_count(&self) -> usize {
+        let mut cnt = 0;
         match self {
-            Node::Text(_) => *current += 1,
+            Node::Text(_) => (),
             Node::Element(element) => {
-                *current += 1;
                 for child in element.children.iter() {
-                    child.node_count_recursive(current);
+                    cnt += child.node_count();
                 }
             }
         }
+        cnt
     }
 
     /// remove the existing attributes and set with the new value
@@ -287,6 +301,17 @@ where
 }
 
 /// create a virtual node with tag, attrs and children
+/// # Example
+/// ```rust
+/// use mt_dom::{Node,element,attr};
+///
+/// let div:Node<&'static str, &'static str, &'static str, &'static str> =
+///     element(
+///          "div",
+///          vec![attr("class", "container")],
+///          vec![],
+///      );
+/// ```
 #[inline]
 pub fn element<NS, TAG, ATT, VAL>(
     tag: TAG,
@@ -303,6 +328,19 @@ where
 }
 
 /// create a virtual node with namespace, tag, attrs and children
+/// # Example
+/// ```rust
+/// use mt_dom::{Node,element_ns,attr};
+///
+/// let svg:Node<&'static str, &'static str, &'static str, &'static str> =
+///     element_ns(
+///         Some("http://www.w3.org/2000/svg"),
+///          "svg",
+///          vec![attr("width","400"), attr("height","400")],
+///          vec![],
+///          false
+///      );
+/// ```
 pub fn element_ns<NS, TAG, ATT, VAL>(
     namespace: Option<NS>,
     tag: TAG,
@@ -319,7 +357,14 @@ where
     Node::Element(Element::new(namespace, tag, attrs, children, self_closing))
 }
 
-/// Create a textnode element
+/// Create a text node element
+/// # Example
+/// ```rust
+/// use mt_dom::{Node,text};
+///
+/// let txt:Node<&'static str, &'static str, &'static str, &'static str> =
+///     text("This is a text node");
+/// ```
 pub fn text<S, NS, TAG, ATT, VAL>(s: S) -> Node<NS, TAG, ATT, VAL>
 where
     S: ToString,
@@ -330,3 +375,25 @@ where
 {
     Node::Text(Text::new(s))
 }
+
+/// Create an html and instruct the DOM renderer and/or DOM patcher that the operation is safe.
+///
+/// Note: this operation doesn't sanitize the html code. It is your responsibility
+/// as a programmer to sanitize the input here.
+/// # Example
+/// ```rust
+/// use mt_dom::{Node,safe_html};
+///
+/// let html: Node<&'static str, &'static str, &'static str, &'static str> =
+///     safe_html("This contains safe html and it&#x27;s solid &nbsp should work correctly");
+/// ```
+pub fn safe_html<S, NS, TAG, ATT, VAL>(s: S) -> Node<NS, TAG, ATT, VAL>
+where
+    S: ToString,
+    NS: PartialEq + Clone + Debug,
+    TAG: PartialEq + Clone + Debug,
+    ATT: PartialEq + Clone + Debug,
+    VAL: PartialEq + Clone + Debug,
+{
+    Node::Text(Text::safe_html(s))
+}
